<?php

namespace Drupal\mask\Helper;

use Drupal\Core\Form\FormStateInterface;

/**
 * Helper class to process Form API elements that support the #mask attribute.
 */
class ElementHelper implements ElementHelperInterface {

  /**
   * {@inheritdoc}
   */
  public function elementInfoAlter(&$info, array $defaults = []) {
    $defaults += [
      'value' => '',
      'reverse' => FALSE,
      'clearifnotmatch' => FALSE,
      'selectonfocus' => FALSE,
    ];
    $info['#mask'] = $defaults;
    $info['#process'][] = [__CLASS__, 'processElement'];
  }

  /**
   * {@inheritdoc}
   */
  public static function processElement(array &$element, FormStateInterface $form_state, array &$complete_form) {
    if (!empty($element['#mask']['value'])) {
      // Adds HTML attributes to the element.
      foreach ($element['#mask'] as $option => $value) {
        if (!empty($value)) {
          $attr = 'data-mask';
          if ($option != 'value') {
            // Appends the option name to the attribute name.
            $attr .= "-$option";

            // Except for the mask, all other options are flags.
            $value = $value ? 'true' : 'false';
          }
          $element['#attributes'][$attr] = is_string($value) ? $value : (string) $value;
        }
      }

      // Attaches the JavaScript library.
      $element['#attached']['library'][] = 'mask/mask_plugin';
    }
    return $element;
  }

}
