<?php

namespace Drupal\mask\Plugin;

use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Field\WidgetInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\NestedArray;

/**
 * Plugin for field widgets that support Mask's settings.
 */
class FieldWidgetPlugin extends PluginBase implements FieldWidgetPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function getWidgetType() {
    return $this->getPluginId();
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldWidgetThirdPartySettings(WidgetInterface $widget) {
    return $widget->getThirdPartySettings('mask') + $this->pluginDefinition['defaults'];
  }

  /**
   * {@inheritdoc}
   */
  public function fieldWidgetThirdPartySettingsForm(WidgetInterface $plugin, FieldDefinitionInterface $field_definition, $form_mode, $form, FormStateInterface $form_state) {
    $mask_settings = $this->getFieldWidgetThirdPartySettings($plugin);
    $element = [
      '#type' => 'fieldset',
      '#title' => $this->t('Mask settings'),
    ];
    $element['value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mask'),
      '#description' => $this->t('The mask value. Please refer to the <a href="@url" target="_blank">jQuery Mask Plugin documentation</a> for information on how to write it. Leave it blank to apply no mask.', [
        '@url' => 'https://igorescobar.github.io/jQuery-Mask-Plugin/docs.html',
      ]),
      '#default_value' => $mask_settings['value'],
    ];
    $element['reverse'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Reverse'),
      '#description' => $this->t('Applies the mask in reverse order (from right to left).'),
      '#default_value' => $mask_settings['reverse'],
    ];
    $element['clearifnotmatch'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Clear if not match'),
      '#description' => $this->t('Clears the input field on submission if the mask is not complete.'),
      '#default_value' => $mask_settings['clearifnotmatch'],
    ];
    $element['selectonfocus'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Select on focus'),
      '#description' => $this->t('Selects the input field value when it is focused.'),
      '#default_value' => $mask_settings['selectonfocus'],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldWidgetSettingsSummaryAlter(&$summary, $context) {
    $widget = $context['widget'];
    $mask_settings = $this->getFieldWidgetThirdPartySettings($widget);
    if ($mask_settings['value']) {
      $summary[] = $this->t('Mask: @value', ['@value' => $mask_settings['value']]);

      $value = $mask_settings['reverse'] ? $this->t('Yes') : $this->t('No');
      $summary[] = $this->t('Reverse: @value', ['@value' => $value]);

      $value = $mask_settings['clearifnotmatch'] ? $this->t('Yes') : $this->t('No');
      $summary[] = $this->t('Clear if not match: @value', ['@value' => $value]);

      $value = $mask_settings['selectonfocus'] ? $this->t('Yes') : $this->t('No');
      $summary[] = $this->t('Select on focus: @value', ['@value' => $value]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function fieldWidgetFormAlter(&$element, FormStateInterface $form_state, $context) {
    $widget = $context['widget'];
    $mask_settings = $this->getFieldWidgetThirdPartySettings($widget);
    if (!empty($mask_settings['value'])) {
      $parents = $this->pluginDefinition['element_parents'];
      $parents[] = '#mask';
      NestedArray::setValue($element, $parents, $mask_settings);
    }
  }

}
