<?php

namespace Drupal\Tests\mask\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\mask\Helper\ElementHelper;
use Drupal\Core\Form\FormState;

/**
 * @coversDefaultClass \Drupal\mask\Helper\ElementHelper
 * @group mask
 */
class ElementHelperTest extends UnitTestCase {

  /**
   * Tests if proper default values are added to element info.
   */
  public function testInfoAlter() {
    $element = [
      '#type' => 'textfield',
    ];

    $helper = new ElementHelper();
    $helper->elementInfoAlter($info);

    $this->assertArraySubset([
      '#mask' => [
        'value' => '',
        'reverse' => FALSE,
        'clearifnotmatch' => FALSE,
        'selectonfocus' => FALSE,
      ],
    ], $info);
    $this->assertContains([get_class($helper), 'processElement'], $info['#process']);
  }

  /**
   * Tests if attributes are added to element that has "#mask" set.
   */
  public function testProcessElement() {
    $element = [
      '#type' => 'textfield',
      '#mask' => [
        'value' => '099.099.099.099',
        'reverse' => TRUE,
      ],
    ];

    $form = [];
    ElementHelper::processElement($element, new FormState(), $form);

    $this->assertArraySubset([
      'data-mask' => '099.099.099.099',
      'data-mask-reverse' => 'true',
    ], $element['#attributes']);
    $this->assertContains('mask/mask_plugin', $element['#attached']['library']);
  }

  /**
   * Tests if attributes are not added to the element if the mask value is not provided.
   */
  public function testProcessElementNoValue() {
    $element = [
      '#type' => 'textfield',
      '#mask' => [
        'reverse' => TRUE,
      ],
      '#attributes' => [],
    ];

    $form = [];
    ElementHelper::processElement($element, new FormState(), $form);

    $this->assertArrayNotHasKey('data-mask', $element['#attributes']);
    $this->assertArrayNotHasKey('data-mask-reverse', $element['#attributes']);
  }

}
