<?php

namespace Drupal\Tests\mask\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\mask\Plugin\FieldWidgetPlugin;
use Drupal\Core\Field\Plugin\Field\FieldWidget\StringTextfieldWidget;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Form\FormState;

/**
 * @coversDefaultClass Drupal\mask\Plugin\FieldWidgetPlugin
 * @group mask
 */
class FieldWidgetPluginTest extends UnitTestCase {

  private $plugin;

  /**
   * Tests if mask settings are added to the field widget form.
   */
  public function testFieldWidgetThirdPartySettingsForm() {
    $widget = $this->createWidget();
    $field_definition = new BaseFieldDefinition([]);

    $element = $this->plugin->fieldWidgetThirdPartySettingsForm($widget, $field_definition, 'default', [], new FormState());

    $this->assertArrayHasKey('value', $element);
    $this->assertArrayHasKey('reverse', $element);
    $this->assertArrayHasKey('clearifnotmatch', $element);
    $this->assertArrayHasKey('selectonfocus', $element);
  }

  /**
   * Tests if the #mask property is added to the proper element.
   */
  public function testFieldWidgetFormAlter() {
    $element = [
      '#type' => 'container',
      'value' => [
        '#type' => 'textfield',
      ],
    ];
    $widget = $this->createWidget([
      'mask' => [
        'value' => '00/00/0000',
        'clearifnotmatch' => TRUE,
      ],
    ]);
    $context['widget'] = $widget;

    $this->plugin->fieldWidgetFormAlter($element, new FormState(), $context);

    $this->assertArraySubset([
      '#mask' => [
        'value' => '00/00/0000',
        'clearifnotmatch' => TRUE,
      ],
    ], $element['value']);
  }

  /**
   * Creates a field widget with the provided third-party settings.
   */
  protected function createWidget($third_party_settings = []) {
    $field_definition = new BaseFieldDefinition([]);
    $widget = new StringTextfieldWidget('string_textfield', [], $field_definition, [], $third_party_settings);
    return $widget;
  }

  /**
   * Instantiates a plugin for the tests.
   */
  protected function setUp() {
    $definition = [
      'defaults' => [
        'value' => '',
        'reverse' => FALSE,
        'clearifnotmatch' => FALSE,
        'selectonfocus' => FALSE,
      ],
      'element_parents' => [ 'value' ],
    ];
    $this->plugin = new FieldWidgetPlugin([], 'string_textfield', $definition);
    $string_translation = $this->getStringTranslationStub();
    $this->plugin->setStringTranslation($string_translation);
  }

}
